@@ -37,11 +37,8 @@ static void usage(void)
 {
 	fprintf(stderr,
 		"Usage: ip mroute show [ [ to ] PREFIX ] [ from PREFIX ] [ iif DEVICE ]\n"
-	"			[ table TABLE_ID ]\n"
-	"TABLE_ID := [ local | main | default | all | NUMBER ]\n"
-#if 0
-	"Usage: ip mroute [ add | del ] DESTINATION from SOURCE [ iif DEVICE ] [ oif DEVICE ]\n"
-#endif
+		"                      [ table TABLE_ID ]\n"
+		"TABLE_ID := [ local | main | default | all | NUMBER ]\n"
 	);
 	exit(-1);
 }
@@ -322,14 +319,7 @@ int do_multiroute(int argc, char **argv)
 {
 	if (argc < 1)
 		return mroute_list(0, NULL);
-#if 0
-	if (matches(*argv, "add") == 0)
-		return mroute_modify(RTM_NEWADDR, argc-1, argv+1);
-	if (matches(*argv, "delete") == 0)
-		return mroute_modify(RTM_DELADDR, argc-1, argv+1);
-	if (matches(*argv, "get") == 0)
-		return mroute_get(argc-1, argv+1);
-#endif
+
 	if (matches(*argv, "list") == 0 || matches(*argv, "show") == 0
 	    || matches(*argv, "lst") == 0)
 		return mroute_list(argc-1, argv+1);
