@@ -19,7 +19,7 @@
 #include <netinet/in.h>
 #include <string.h>
 
-#include "SNAPSHOT.h"
+#include "version.h"
 
 #include "utils.h"
 #include "libnetlink.h"
@@ -65,7 +65,7 @@ static void usage(void)
 	fprintf(stderr,
 		"Usage: rtmon [ OPTIONS ] file FILE [ all | LISTofOBJECTS ]\n"
 		"OPTIONS := { -f[amily] { inet | inet6 | link | help } |\n"
-		"	     -4 | -6 | -0 | -V[ersion] }\n"
+		"             -4 | -6 | -0 | -V[ersion] }\n"
 		"LISTofOBJECTS := [ link ] [ address ] [ route ]\n");
 	exit(-1);
 }
@@ -107,7 +107,7 @@ main(int argc, char **argv)
 		} else if (strcmp(argv[1], "-0") == 0) {
 			family = AF_PACKET;
 		} else if (matches(argv[1], "-Version") == 0) {
-			printf("rtmon utility, iproute2-ss%s\n", SNAPSHOT);
+			printf("rtmon utility, iproute2-%s\n", version);
 			exit(0);
 		} else if (matches(argv[1], "file") == 0) {
 			argc--;
