@@ -26,20 +26,26 @@
 #include "rt_names.h"
 #include "utils.h"
 
-
-const char *ll_addr_n2a(const unsigned char *addr, int alen, int type, char *buf, int blen)
+const char *ll_addr_n2a(const unsigned char *addr, int alen, int type,
+			char *buf, int blen)
 {
 	int i;
 	int l;
 
 	if (alen == 4 &&
-	    (type == ARPHRD_TUNNEL || type == ARPHRD_SIT || type == ARPHRD_IPGRE)) {
+	    (type == ARPHRD_TUNNEL || type == ARPHRD_SIT
+	     || type == ARPHRD_IPGRE))
 		return inet_ntop(AF_INET, addr, buf, blen);
-	}
-	if (alen == 16 &&
-	    (type == ARPHRD_TUNNEL6 || type == ARPHRD_IP6GRE)) {
+
+	if (alen == 16 && (type == ARPHRD_TUNNEL6 || type == ARPHRD_IP6GRE))
 		return inet_ntop(AF_INET6, addr, buf, blen);
-	}
+	if (alen == 7 && type == ARPHRD_AX25)
+		return ax25_ntop(AF_AX25, addr, buf, blen);
+	if (alen == 7 && type == ARPHRD_NETROM)
+		return netrom_ntop(AF_NETROM, addr, buf, blen);
+	if (alen == 5 && type == ARPHRD_ROSE)
+		return rose_ntop(AF_ROSE, addr, buf, blen);
+
 	snprintf(buf, blen, "%02x", addr[0]);
 	for (i = 1, l = 2; i < alen && l < blen; i++, l += 3)
 		snprintf(buf + l, blen - l, ":%02x", addr[i]);
@@ -62,7 +68,7 @@ int ll_addr_a2n(char *lladdr, int len, const char *arg)
 	} else {
 		int i;
 
-		for (i=0; i<len; i++) {
+		for (i = 0; i < len; i++) {
 			int temp;
 			char *cp = strchr(arg, ':');
 			if (cp) {
@@ -70,11 +76,13 @@ int ll_addr_a2n(char *lladdr, int len, const char *arg)
 				cp++;
 			}
 			if (sscanf(arg, "%x", &temp) != 1) {
-				fprintf(stderr, "\"%s\" is invalid lladdr.\n", arg);
+				fprintf(stderr, "\"%s\" is invalid lladdr.\n",
+					arg);
 				return -1;
 			}
 			if (temp < 0 || temp > 255) {
-				fprintf(stderr, "\"%s\" is invalid lladdr.\n", arg);
+				fprintf(stderr, "\"%s\" is invalid lladdr.\n",
+					arg);
 				return -1;
 			}
 			lladdr[i] = temp;
@@ -82,6 +90,6 @@ int ll_addr_a2n(char *lladdr, int len, const char *arg)
 				break;
 			arg = cp;
 		}
-		return i+1;
+		return i + 1;
 	}
 }
